function mImageRGB = ReshapeToRGBF(cImageFolded,cIndexFolded, blockWidth, blockHeight, ...
    vPlainText, maxIntensity, approxType, adj, meanIntensity,vIXBlockEnd, ...
    vIXBlockEndI)

% Takes a matrix of pixel intensities and reshapes to a three dimensional
% array representing an RGB intensity image.
%   Only uses blocks that contain info, established using iLastBlock.
%   Includes space for the plain text information, which includes resizing
% parameters generated by this function.
%   approxType determins the number of plaintext parameters recovered and
% the way the approximation will be recovered.


% Flatten the image to a 1D vector
% <<--
    vFlat = Flatten(cImageFolded,blockHeight,blockWidth,...
        vIXBlockEnd);
% -->>

% Flatten the indices to a 1D vector
    vIndex = Flatten(cIndexFolded,blockHeight,blockWidth,...
        vIXBlockEndI);
% -->> 

% Join the image and indexes
    vFlat = [vFlat vIndex];
% - - > > #

% Add padding parameter to sPlainText
%   If numel(vImage) cannot be exactly divided by 3 padding may need to be
% added to the end, the padding variable stores the amount of padding.
% <<--
% At this point the amount of padding has not been determined so initialize
% to zero.
    padding = 0;
% -->>

% Number of plain text parameters
%   +5 to include the padding mult and diff of nPlainText, approxType and
%   plain text adjustment value (will be less than max intensity to save)
% <<--
    nPlainText = numel(vPlainText)+5;
% -->>

% Number of pixels in new image if padding is not required
% <<--
    nPixels = numel(vFlat) + nPlainText;
% -->>


% Calculate the first 2 dimensions of mImageRGB
%   Padding may be required if nPixels is not exactly divisible by 3.
% <<--
% As this needs to be as close to the number of columns in mImageGrey this
% is used in the calculation.
    nColumns = size(cImageFolded{1},2);
    [nColumnsRGB nRowsRGB padding] = CalcDimRGB(nPixels,nColumns);
% -->>


% Add padding if required
% <<--
if padding
    
    % Make the padding light Blue, i.e. half the max intensity as padding will appear
    % in the last (Blue) colour plane.
    %   This is to avoid the padding appearing as a yellow block (RG).
    % vImage = [vImage, 0.5*maxIntensity*ones(1,padding)];
    % Set the padding the the mean intensity of the folded image
    vFlat = [vFlat, meanIntensity*ones(1,padding)];
        
end
% -->>

% Store nPlainText as multiples of maxIntensity
% <<--
    [multNPlainText diffNPlainText] = Reduce(nPlainText,maxIntensity);
% -->>

% Add the plain text parameters 
%   These are the parameters that need to be recovered exactly and are
% therefore stored as pixels in the mImageRGB
%   Performed here so they can be taken from the image without requiring
% the amount of padding used.
% <<--
    %vImage = AddPlainText(vImage,sPlainText);
    % Add the padding parameter and the number of plain text parameters
    vFlat = [vFlat vPlainText padding multNPlainText diffNPlainText...
        approxType adj];
% -->>

% Reshape vImage to have 3 dimensions
% <<--
    mImageRGB = reshape(vFlat,nRowsRGB,nColumnsRGB,3);
% -->>


end



function [mult diff] = Reduce(n,sz)

    % Reduct n to multiples of size plus a difference so n can be recovered
    % as n = mult*size + diff;
    
    mult = fix(n/sz);
    diff = n - mult*sz;

end

function [nColumnsRGB nRowsRGB padding] = CalcDimRGB(nPixels,nColumns)

    % Calculates the dimensions of a 3 dimensional array that contains
    % nPixels.  
    % The only constraint is that is should have as near to nColumns as 
    % possible.
    % UPDATE - For simplicity should maintain the same number of columns as
    % the original and pad to get the same number of rows.
    % Padding may be required if nPixels is not divisible by 3.
        
    
    % Number of pixels per colour plane
    nPixelsPerPlane = nPixels/3;
    
    
    % Is padding required
    %   If so how much for the whole image, as this will be applied to the
    % last (Blue) colour plane.
    %   What is the new value of nPixelsPerPlane
    % <<--
    % Padding is zero if not required
    padding = 0;    
    if ceil(nPixelsPerPlane) ~= nPixelsPerPlane
        
        % The new image has to be at least as big as the eixisting one,
        % therefore use ceil.
        nPixelsPerPlane = ceil(nPixelsPerPlane);
        
        % How much padding
        padding = nPixelsPerPlane*3-nPixels;
        
    end
    % -->>
    
    
    % Number of columns in RGB is the same as the original
    % <<--
    nColumnsRGB = nColumns;
    % -->>    
    
    % Number of columns in mImageRGB
    % <<--
    nRowsRGB = ceil(nPixelsPerPlane/nColumns);
    % -->>

    % If the new dimensions require more pixels add to padding
    % <<--
    
        % Padding required in one pixel plain
        OnePlanePadding = (nColumnsRGB*nRowsRGB - nPixelsPerPlane);
        
        % 3*OnePlainPadding to add padding to the entire image.
        padding = padding + 3*OnePlanePadding;
        
    % -->>
    
end

%{
function vImage = Flatten(cImage,blockHeight,blockWidth,vIXBlockEnd)

    % Takes an mxn matrix and returns a 1 x nPixels vector containing all
    % the folded pixels in mImage.
    %   Ignores any pixels in the last blockHeight rows of the any plain in the image
    % image in
    % columns greater than vIXBlockEnd x blockWidth. i.e. the new matrix
    % only includes folded blocks and not padded zero blocks.
    %   Assumes mImage can be exactly blocked as it is the product of
    % previous blocking routine.
    
    nColours = numel(cImage);
    mFlat = [];
    
    % For each colour plain store the pixels
    for iColour = 1:nColours
            
        % Get current number of rows
        %   nColumns is not required.
        % <<--
            nRows = size(cImage{iColour},1);    
        % -->>

        % Number of useful columns in the last row of mImage and mIndex.
        %   Any pixels in the last blockHeight rows extending beyond this
        % column contain zeros used as padding.
        % <<--
            nColumnsRemaining = vIXBlockEnd(iColour)*blockWidth;
        % -->>

        % Number of row of mImage with full columns
        % <<--
            nRowsFull = nRows - blockHeight;
        % -->>

        % Flatten the image to be blockHeight x N
        %  The size of N is not important, reshape calculates this internally,
        % it is not required by the rest of the routine.
        % <<--

        % First take all the full rows
            mRowsFull = cImage{iColour}(1:nRowsFull,:);

        % Flatten mRowsFull to height blockHeight
            mFlat = [mFlat reshape(mRowsFull,blockHeight,[])];

        % Get the pixels containing folded data in the last blockHeight rows of
        % mImage.
            mLastBlocks = cImage{iColour}(nRowsFull+1:end,1:nColumnsRemaining);

        % Place the last nColumnsRemaining at the end of mImageFlat
            mFlat = [mFlat, mLastBlocks];

        % -->>
    end
    
            
    % Flatten mImageFlat to be a row vector
    % <<--
    % : has the same effect as using reshape except it will always return a
    % column vector, therefore transpose.
    vImage = mFlat(:)';
    % -->>

end
%}