% SCEIF_Example This example demonstrates the use of the Self Contained
% Encrypted Image Folding Software (SCEIF) by reproducing an example in the 
% paper:
%
%  [1]  Self Contained Encrypted Image Folding
%
% The SCEIF method is applied to the image before saving as raw and 
% clearing the workspace.
% The recovery is then performed and the results displayed using the
% correct and incorrect private keys.
%
% OPTIONAL:
% 1) The included test image is the image of the nebula NGC 2264 stored as 
% a raw file.  To use your own image change the imagePath variable, line 33
% to the path where your image is stored.  If the image is not in .raw 
% format uncomment line 38 to convert it to raw.
% 2) The approximation quality can be changed by altering the PSNRApprox
% variable (line 62).  This is the desired PSNR in the approximation.
% 3) For a much faster implementation please use the C++ implementations of 
% OMP2DMl and OMP2D.  For this run the CompileMex.m script.
% 4) The default algorithm OMPMl2DW processes the three colour plains
% (R,G,B) together returning a single set of indexes and three sets of
% coefficients. This can be changed to OMPMl2DW to process each colour
% plain seperatly by uncommenting line 50.

% Clear all variables from the workspace before starting
    close all;
    clear all;
    clc;
%% Approximation Variables

% Path to image
    imagePath = 'red_astro.raw';  
% - - > > #

% If using a standard image file format uncomment below to convert to
% raw.
%     imagePath = ImageTo16BitRaw(imagePath);
% - - > > #    
    
% Details of the dicitonary used in the approximation
    sApproxDetails.approxMethod = 'Dictionary';
    sApproxDetails.sDetails = struct('name',{'DCT','Hats','Cubic','Cubic'...
        },'parameters',{2,1,1,2});    
 % - - > > #   
     
% Approximation algorithm
%   Uncomment desired alg
    algorithm = 'OMP2DMlW';   
%     algorithm = 'OMP2DW';
% - - > > #   
     
% Public key to prevent plain text attacks
    publicKey = 19;
% - - > > #        

% Save known variables
    save known_vars sApproxDetails algorithm publicKey
% - - > > #

% Desired approximation quality
    PSNRApprox = 39.565; 
% - - > > #
%% Image Approximation

% Approximate the image
    sImageApprox = SImageApproxSCEIFS(imagePath,PSNRApprox,'greedy2D',algorithm,...
        sApproxDetails);
    approxTime = sImageApprox.approxTime
% - - > > #
    
% Save summary vars
    save summary_vars sImageApprox approxTime
% - - > > #    
%% Fold the image

% Encryption Variable
    privateKey = 1234567891;
% - - > > #
    
% Fold the image
    startTime = clock;
    mImageFolded = Folding(sImageApprox,sApproxDetails,publicKey,privateKey);
    foldTime = etime(clock,startTime)
% - - > > #     

% Save variables required for summary information
    save('summary_vars','foldTime','mImageFolded','-append');
% - - > > # 

% Save the image as a lossless png file
%   Use PNG as this supports 48 bit truecolour
    saveImagePath = 'SCEIF_folded.raw';
    savePrecision = 'uint16';
    SaveImageRaw(mImageFolded,saveImagePath,savePrecision);
% - - > > # 

% Clear workspace to make sure all needed info is in the saved file
    clear all;
% - - > > #     
%% Expand using the incorrect private key

% Load the folded image
    loadImagePath = 'SCEIF_folded.raw';
    savePrecision = 'uint16';
    mImageLoaded = double(LoadImageRaw(loadImagePath,savePrecision));
% - - > > # 

% Encryption Variables
    privateKey = 1234567890; % Incorrect private key
% - - > > # 

% Recover the image
    % Load required know variables
        sKnown = load('known_vars','sApproxDetails','publicKey');
    % - - > >
    
    [mImageRecovered1 cImage] = Expand(mImageLoaded,sKnown.sApproxDetails,...
        sKnown.publicKey,privateKey);
% - - > > #     

% Save variables required for summary information
    save('summary_vars','mImageRecovered1','-append');
% - - > > #

% Display figures
    % Load the original image
        load('summary_vars','sImageApprox');
    % - - > > 
    
    % Display fig 2 from the paper
        ShowFig2(RUint16(sImageApprox.mImage));
    % - - > >
    
    % Display fig 3 from the paper               
        ShowFig3(uint16(mImageLoaded),...
            cellfun(@RUint16,cImage,'UniformOutPut',0),...
            uint16(mImageRecovered1),'Incorrect Private Key');
    % - - > >
% - - > > #

% Report quality of the recovery from the folding procedure
    fprintf('\nQuality of the recovery (Original Image / Recovered Plain Text Image)\n')
    fprintf('Incorrect Key: ');
    PrintPSNR(sImageApprox.mImage,mImageRecovered1,sImageApprox.maxIntensity);
% - - > > #

% Clear workspace to make sure all needed info is in the saved file
    clear all;
% - - > > #
%% Expand using the correct private key

% Load the folded image
    loadImagePath = 'SCEIF_folded.raw';
    savePrecision = 'uint16';
    mImageLoaded = double(LoadImageRaw(loadImagePath,savePrecision));
% - - > > #

% Encryption Variable
    privateKey = 1234567891; % Correct private key
% - - > > #

% Recover the image
    startTime = clock;
    % Load required know variables
        sKnown = load('known_vars','sApproxDetails','publicKey');
    % - - > > #
    [mImageRecovered2 cImage] = Expand(mImageLoaded,sKnown.sApproxDetails,...
        sKnown.publicKey,privateKey);
    expandTime = etime(clock,startTime);
% - - > > #

% Save variables required for summary information
    save('summary_vars','expandTime','mImageRecovered2','-append');
% - - > > #
   
% Report quality of the recovery from the folding procedure
    % Load the original image
        load('summary_vars','sImageApprox');
    % - - > > 
    fprintf('\nQuality of the recovery (Original Image / Recovered Plain Text Image)\n')
    fprintf('Correct Key: ');
    PrintPSNRSSIM(sImageApprox.mImage,mImageRecovered2,sImageApprox.maxIntensity);
% - - > > #

% Display fig 3 from the paper
    ShowFig3(uint16(mImageLoaded),...
    cellfun(@RUint16,cImage,'UniformOutPut',0),...
    uint16(mImageRecovered2),'Correct Private Key');
% - - > > #
%% Print Summary

% Load the summary info
    load summary_vars
% - - > > #

% Print summary of approx
    fprintf('\n--------------------------------------------------------------------\n');
    PrintWithStars('Summary');
    fprintf('Quaility of the approximation (Original Image / Plain Text Image)\n')
    PrintPSNRSSIM(sImageApprox.mImage,sImageApprox.mImageApprox,...
        sImageApprox.maxIntensity);
    fprintf('Quality of the recovery (Original Image / Recovered Plain Text Image)\n')
    fprintf('Correct Key:   ');
    PrintPSNRSSIM(sImageApprox.mImage,mImageRecovered2,...
        sImageApprox.maxIntensity);
    fprintf('Incorrect Key: ');
    PrintPSNR(sImageApprox.mImage,mImageRecovered1,sImageApprox.maxIntensity);
    PrintWithStars('Image Dimensions');
    fprintf('Plain Text Image\n');
    fprintf('%4i x %4i x %1i\n',sImageApprox.y,sImageApprox.x,sImageApprox.z);
    fprintf('Folded Image\n');
    fprintf('%4i x %4i x %1i\n',size(mImageFolded));
    PrintWithStars('Execution Times');
    fprintf('Approximation: %.2f seconds (Building the Plain Text Image)\n',approxTime);
    fprintf('Folding:       %.2f seconds\n',foldTime);
    fprintf('Expanding:     %.2f seconds\n',expandTime);
    fprintf('Total Time:    %.2f seconds\n',approxTime+foldTime+expandTime);
    fprintf('\n--------------------------------------------------------------------\n')    
% - - > > #